@@ -13,16 +13,16 @@
 ##############################################################################
 
 """
-$Id: interfaces.py 114890 2010-07-21 12:18:55Z adamg $
+$Id: interfaces.py 114887 2010-07-21 12:06:15Z adamg $
 """
 __docformat__ = 'restructuredtext'
 
 import zope.interface
+import zope.i18nmessageid
 import zope.schema
 from zope.schema.interfaces import IVocabularyTokenized
-from zope.session.interfaces import ISession
-from zope.i18n.interfaces import INegotiator
-from z3c.i18n import MessageFactory as _
+
+_ = zope.i18nmessageid.MessageFactory('z3c')
 
 
 language_policies = ['server', 'session', 'browser',
